#!/bin/sh
# $Id: MACwatch.tcl,v 1.7 2001/02/06 16:17:20 gdmr Exp $

# Copyright (c) 2001 Division of Informatics, University of Edinburgh
# 
# Permission to use, copy, modify, distribute, and sell this software and its
# documentation for any purpose is hereby granted without fee, provided that
# the above copyright notice appear in all copies and that both that
# copyright notice and this permission notice appear in supporting
# documentation, and that the name of the University of Edinburgh not be used
# in advertising or publicity pertaining to distribution of the software
# without specific, written prior permission.  The University of Edinburgh
# makes no representations about the suitability of this software for any
# purpose.  It is provided "as is" without express or implied warranty.
#
# Comments are invited.  Please send to gdmr@dcs.ed.ac.uk.

# tclsh is in a different place on Solaris and Linux, and we can't
# assume that it's in our PATH as we may be running from a cron job.
# This is the standard tcl hack, only with more than one shell line...
#\
case "`uname -sr`" in
#\
	'SunOS 5'*)	TCLSH=/usr/local/bin/tclsh ;;
#\
	Linux*)		TCLSH=/usr/tcl8/bin/tclsh ;;
#\
	 *)		echo "Don't know where to find tclsh" ; exit ;;
#\
esac
#\
exec $TCLSH "$0" "$@"
# It's tcl from here on...

# Auto-load from the directory that this script lives in
lappend auto_path [file dirname $argv0]

set community "dcsHubRead"
set history "MAChistory"

# Current timestamp
set now [exec date]
puts stderr "Now is $now"

# Read and write the history file
proc LoadHistory {} {
   global history
   global MACs
   puts stderr "LoadHistory"
   if [file exists $history] {
      if [catch {source $history} problem] {
         puts stderr "Couldn't load history: $problem"
         exit 1
      }
   } else {
      puts "No $history (yet)"
   }
}

proc WriteHistory {} {
   global history
   global MACs
   puts stderr "WriteHistory"
   if [catch {open $history.new "w"} h] {
      puts "Couldn't write $history.new: $h"
      return
   }
   foreach i [lsort [array names MACs]] {
      puts $h "set MACs($i) \"$MACs($i)\""
   }
   if [file exists $history] {
      file rename -force $history $history.old
   }
   file rename $history.new $history
}

# Grab a switch's forwarding table
proc GrabTable {s} {
   global community
   global MACs
   global now
   puts stderr "GrabTable <$s>"
   set snmpS [snmpInit $s $community]
   if [catch {snmpPortFwdTable $snmpS} pft] {
      puts stderr "$s not responding: $pft"
      return
   }
   foreach p $pft {
      set pa [string tolower [lindex $p 0]]
      set pp [lindex $p 1]
      set f "$pa/$s/$pp/first"
      set l "$pa/$s/$pp/last"
      if {![info exists MACs($f)]} {
         set MACs($f) $now
      }
      set MACs($l) $now
   }
}

LoadHistory
foreach a $argv {
   GrabTable $a
}
WriteHistory

exit 0
